package de.szut.casino.slots;

import de.szut.casino.shared.dto.BetDto;
import de.szut.casino.shared.service.BalanceService;
import de.szut.casino.user.UserEntity;
import de.szut.casino.user.UserService;
import jakarta.validation.Valid;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RestController;

import java.util.Collections;
import java.util.Optional;

@RestController
public class SlotController {
    private final UserService userService;
    private final BalanceService balanceService;
    private final SlotService slotService;

    public SlotController(UserService userService, BalanceService balanceService, SlotService slotService) {
        this.userService = userService;
        this.balanceService = balanceService;
        this.slotService = slotService;
    }

    @PostMapping("/slots/spin")
    public ResponseEntity<Object> spinSlots(@RequestBody @Valid BetDto betDto, @RequestHeader("Authorization") String token) {
        Optional<UserEntity> optionalUser = userService.getCurrentUser(token);

        if (optionalUser.isEmpty()) {
            return ResponseEntity.notFound().build();
        }

        UserEntity user = optionalUser.get();

        if (!this.balanceService.hasFunds(user, betDto)) {
            return ResponseEntity.badRequest().body(Collections.singletonMap("error", "Insufficient funds"));
        }

        SpinResult spinResult = this.slotService.spin(
                betDto.getBetAmount(),
                user
        );

        return ResponseEntity.ok(spinResult);
    }
}
